@@ -1,5 +1,5 @@
 # noinspection PyUnresolvedReferences
-from zuper_nodes import InteractionProtocol, particularize
+from zuper_nodes import InteractionProtocol, particularize, particularize_no_check
 
 # noinspection PyUnresolvedReferences
 from zuper_nodes_wrapper import wrap_direct, Context
