@@ -1,6 +1,7 @@
 from typing import Dict, TYPE_CHECKING
 
 import numpy as np
+
 from zuper_nodes import particularize_no_check
 
 if TYPE_CHECKING:
@@ -12,6 +13,7 @@ from .protocol_agent import protocol_agent
 from .protocol_simulator import (
     DuckieState,
     JPGImage,
+    JPGImageWithTimestamp,
     protocol_simulator,
     RobotName,
     RobotObservations,
@@ -47,13 +49,21 @@ __all__ = [
     "protocol_simulator_DB20",
     "DTSimDuckieInfo",
     "DTSimDuckieState",
+    "DB20ObservationsOnlyState",
+    "protocol_agent_DB20_onlystate",
+    "DB20ObservationsOnlyState",
+    "protocol_agent_DB20_timestamps",
+    "protocol_simulator_DB20_timestamps",
+    "DB20ObservationsWithTimestamp",
+    "DB20OdometryWithTimestamp",
+    "DB20RobotObservationsWithTimestamp",
 ]
 
 
 @dataclass
 class PWMCommands:
     """
-        PWM commands are floats between -1 and 1.
+    PWM commands are floats between -1 and 1.
     """
 
     motor_left: float
@@ -75,7 +85,7 @@ class Duckiebot1Observations:
 
 @dataclass
 class RGB:
-    """ Values between 0, 1. """
+    """Values between 0, 1."""
 
     r: float
     g: float
@@ -179,13 +189,17 @@ protocol_agent_duckiebot1 = particularize_no_check(
 
 description = """Particularization for Duckiebot1; observations and commands with full state """
 protocol_agent_duckiebot1_fullstate = particularize_no_check(
-    protocol_agent_duckiebot1, inputs={"observations": Duckiebot1ObservationsPlusState},
+    protocol_agent_duckiebot1,
+    inputs={"observations": Duckiebot1ObservationsPlusState},
 )
 
 protocol_simulator_duckiebot1 = particularize_no_check(
     protocol_simulator,
     description="""Particularization for Duckiebot1 observations and commands.""",
-    inputs={"set_robot_commands": DB18SetRobotCommands, "set_map": DTSetMap,},
+    inputs={
+        "set_robot_commands": DB18SetRobotCommands,
+        "set_map": DTSetMap,
+    },
     outputs={
         "robot_observations": DB18RobotObservations,
         "robot_state": DTSimRobotState,
@@ -207,16 +221,37 @@ class DB20Odometry:
     """" The current rotation of the right wheel. Positive when robot goes forward """
 
 
+@dataclass
+class DB20OdometryWithTimestamp(DB20Odometry):
+    timestamp: float
+
+
 @dataclass
 class DB20Observations:
     camera: JPGImage
     odometry: DB20Odometry
 
 
+@dataclass
+class DB20ObservationsWithTimestamp:
+    camera: JPGImageWithTimestamp
+    odometry: DB20OdometryWithTimestamp
+
+
 @dataclass
 class DB20ObservationsPlusState:
-    camera: JPGImage
-    odometry: DB20Odometry
+    camera: JPGImageWithTimestamp
+    odometry: DB20OdometryWithTimestamp
+
+    your_name: RobotName
+    state: DTSimState
+    map_data: str
+
+
+@dataclass
+class DB20ObservationsOnlyState:
+    # camera: JPGImage
+    # odometry: DB20Odometry
 
     your_name: RobotName
     state: DTSimState
@@ -243,21 +278,43 @@ class DB20RobotObservations(RobotObservations):
     observations: DB20Observations
 
 
+@dataclass
+class DB20RobotObservationsWithTimestamp(RobotObservations):
+    robot_name: RobotName
+    t_effective: float
+    observations: DB20ObservationsWithTimestamp
+
+
 protocol_agent_DB20 = particularize_no_check(
     protocol_agent,
     description="""Particularization for DB20 observations and commands.""",
     inputs={"observations": DB20Observations},
     outputs={"commands": DB20Commands},
 )
+protocol_agent_DB20_timestamps = particularize_no_check(
+    protocol_agent,
+    description="""Particularization for DB20 observations and commands and timestamps.""",
+    inputs={"observations": DB20ObservationsWithTimestamp},
+    outputs={"commands": DB20Commands},
+)
 
 protocol_agent_DB20_fullstate = particularize_no_check(
-    protocol_agent_duckiebot1, inputs={"observations": DB20ObservationsPlusState},
+    protocol_agent_duckiebot1,
+    inputs={"observations": DB20ObservationsPlusState},
+)
+
+protocol_agent_DB20_onlystate = particularize_no_check(
+    protocol_agent_duckiebot1,
+    inputs={"observations": DB20ObservationsOnlyState},
 )
 
 protocol_simulator_DB20 = particularize_no_check(
     protocol_simulator,
     description="""Particularization for Duckiebot1 observations and commands.""",
-    inputs={"set_robot_commands": DB20SetRobotCommands, "set_map": DTSetMap,},
+    inputs={
+        "set_robot_commands": DB20SetRobotCommands,
+        "set_map": DTSetMap,
+    },
     outputs={
         "robot_observations": DB20RobotObservations,
         "robot_state": DTSimRobotState,
@@ -265,3 +322,18 @@ protocol_simulator_DB20 = particularize_no_check(
         "state_dump": DTSimStateDump,
     },
 )
+
+protocol_simulator_DB20_timestamps = particularize_no_check(
+    protocol_simulator,
+    description="""Particularization for Duckiebot1 observations and commands with timestamps""",
+    inputs={
+        "set_robot_commands": DB20SetRobotCommands,
+        "set_map": DTSetMap,
+    },
+    outputs={
+        "robot_observations": DB20RobotObservationsWithTimestamp,
+        "robot_state": DTSimRobotState,
+        "duckie_state": DTSimDuckieState,
+        "state_dump": DTSimStateDump,
+    },
+)
